searchNodes=[{"type":"module","title":"Styler","doc":"Styler is a formatter plugin with stronger opinions on code organization, multi-line defs and other code-style matters.","ref":"Styler.html"},{"type":"behaviour","title":"Styler.Style","doc":"A Style takes AST and returns a transformed version of that AST. Because these transformations involve traversing trees (the &quot;T&quot; in &quot;AST&quot;), we wrap the AST in a structure called a Zipper to facilitate walking the trees.","ref":"Styler.Style.html"},{"type":"function","title":"Styler.Style.displace_comments/2","doc":"Set the line of all comments with line in range_start..range_end to instead have line range_start","ref":"Styler.Style.html#displace_comments/2"},{"type":"function","title":"Styler.Style.empty_map?/1","doc":"Returns true if the ast represents an empty map","ref":"Styler.Style.html#empty_map?/1"},{"type":"function","title":"Styler.Style.ensure_block_parent/1","doc":"Returns the current node (wrapped in a __block__ if necessary) if it's a valid place to insert additional nodes","ref":"Styler.Style.html#ensure_block_parent/1"},{"type":"function","title":"Styler.Style.find_nearest_block/1","doc":"Returns a zipper focused on the nearest node where additional nodes can be inserted (a &quot;block&quot;). The nearest node is either the current node, an ancestor, or one of those two but wrapped in a new :__block__ node.","ref":"Styler.Style.html#find_nearest_block/1"},{"type":"callback","title":"Styler.Style.run/2","doc":"run will be used with Zipper.traverse_while/3 , meaning it will be executed on every node of the AST. You can skip traversing parts of the tree by returning a Zipper that's further along in the traversal, for example by calling Zipper.skip(zipper) to skip an entire subtree you know is of no interest to your Style.","ref":"Styler.Style.html#c:run/2"},{"type":"function","title":"Styler.Style.set_line/3","doc":"Recursively sets :line meta to line . Deletes :newlines unless delete_lines: false is passed","ref":"Styler.Style.html#set_line/3"},{"type":"function","title":"Styler.Style.shift_comments/2","doc":"Perform a series of shifts in a single pass. When shifting comments from block A to block B, naively using two passes of shift_comments/3 would result in all comments ending up in either region A or region B (because A would move to B, then all B back to A) This function exists to make sure that a comment is only moved once during the swap.","ref":"Styler.Style.html#shift_comments/2"},{"type":"function","title":"Styler.Style.shift_comments/3","doc":"Change the line of all comments with line in range by adding delta to it. A positive delta will move the lines further down a file, while a negative delta will move them up.","ref":"Styler.Style.html#shift_comments/3"},{"type":"function","title":"Styler.Style.shift_line/2","doc":"Recursively updates :line meta by adding delta","ref":"Styler.Style.html#shift_line/2"},{"type":"function","title":"Styler.Style.update_all_meta/2","doc":"Traverses an ast node, updating all nodes' meta with meta_fun","ref":"Styler.Style.html#update_all_meta/2"},{"type":"type","title":"Styler.Style.context/0","doc":"","ref":"Styler.Style.html#t:context/0"},{"type":"module","title":"Styler.Style.Blocks","doc":"Simple 1-1 rewrites all crammed into one module to make for more efficient traversals Credo Rules addressed: Credo.Check.Consistency.ParameterPatternMatching Credo.Check.Readability.LargeNumbers Credo.Check.Readability.ParenthesesOnZeroArityDefs Credo.Check.Readability.PreferImplicitTry Credo.Check.Readability.WithSingleClause Credo.Check.Refactor.CaseTrivialMatches Credo.Check.Refactor.CondStatements Credo.Check.Refactor.RedundantWithClauseResult Credo.Check.Refactor.WithClauses","ref":"Styler.Style.Blocks.html"},{"type":"function","title":"Styler.Style.Blocks.run/2","doc":"Callback implementation for Styler.Style.run/2 .","ref":"Styler.Style.Blocks.html#run/2"},{"type":"module","title":"Styler.Style.Defs","doc":"Styles function heads so that they're as small as possible. The goal is that a function head fits on a single line. This isn't a Credo issue, and the formatter is fine with either approach. But Styler has opinions! Ex: This long declaration def foo ( %{ bar : baz } ) do ... end Becomes def foo ( %{ bar : baz } ) do ... end","ref":"Styler.Style.Defs.html"},{"type":"function","title":"Styler.Style.Defs.run/2","doc":"Callback implementation for Styler.Style.run/2 .","ref":"Styler.Style.Defs.html#run/2"},{"type":"module","title":"Styler.Style.ModuleDirectives","doc":"Styles up module directives! This Style will expand multi-aliases/requires/imports/use and sort the directive within its groups (except use s, which cannot be sorted) It also adds a blank line after each directive group. Credo rules Rewrites for the following Credo rules: Credo.Check.Consistency.MultiAliasImportRequireUse (force expansion) Credo.Check.Readability.AliasOrder (we sort __MODULE__ , which credo doesn't) Credo.Check.Readability.ModuleDoc (adds @moduledoc false if missing. includes *.exs files) Credo.Check.Readability.MultiAlias Credo.Check.Readability.StrictModuleLayout (see section below for details) Credo.Check.Readability.UnnecessaryAliasExpansion Strict Layout This can break your code. Modules directives are sorted into the following order: @shortdoc @moduledoc @behaviour use import alias require everything else (unchanged) If any of the sorted directives had a dependency on code that is now below it, your code will fail to compile after being styled. For instance, the following will be broken because the module attribute definition will be moved below the use clause, meaning @pi is undefined when invoked. # before defmodule Approximation do @pi 3.14 use Math , pi : @pi end # after defmodule Approximation do @moduledoc false use Math , pi : @pi @pi 3.14 end For now, it's up to you to come up with a fix for this issue. Sorry!","ref":"Styler.Style.ModuleDirectives.html"},{"type":"function","title":"Styler.Style.ModuleDirectives.run/2","doc":"Callback implementation for Styler.Style.run/2 .","ref":"Styler.Style.ModuleDirectives.html#run/2"},{"type":"module","title":"Styler.Style.Pipes","doc":"Styles pipes! In particular, don't make pipe chains of only one pipe, and some persnickety pipe chain start stuff. Rewrites for the following Credo rules: Credo.Check.Readability.BlockPipe Credo.Check.Readability.OneArityFunctionInPipe Credo.Check.Readability.PipeIntoAnonymousFunctions Credo.Check.Readability.SinglePipe Credo.Check.Refactor.FilterCount Credo.Check.Refactor.MapInto Credo.Check.Refactor.MapJoin Credo.Check.Refactor.PipeChainStart, excluded_functions: [&quot;from&quot;]","ref":"Styler.Style.Pipes.html"},{"type":"function","title":"Styler.Style.Pipes.run/2","doc":"Callback implementation for Styler.Style.run/2 .","ref":"Styler.Style.Pipes.html#run/2"},{"type":"module","title":"Styler.Style.SingleNode","doc":"Simple 1-1 rewrites all crammed into one module to make for more efficient traversals Credo Rules addressed: Credo.Check.Consistency.ParameterPatternMatching Credo.Check.Readability.LargeNumbers Credo.Check.Readability.ParenthesesOnZeroArityDefs Credo.Check.Readability.PreferImplicitTry Credo.Check.Readability.WithSingleClause Credo.Check.Refactor.CaseTrivialMatches Credo.Check.Refactor.CondStatements Credo.Check.Refactor.RedundantWithClauseResult Credo.Check.Refactor.WithClauses","ref":"Styler.Style.SingleNode.html"},{"type":"function","title":"Styler.Style.SingleNode.run/2","doc":"Callback implementation for Styler.Style.run/2 .","ref":"Styler.Style.SingleNode.html#run/2"},{"type":"module","title":"Styler.Zipper","doc":"Implements a Zipper for the Elixir AST based on Gérard Huet Functional pearl: the zipper paper and Clojure's clojure.zip API. A zipper is a data structure that represents a location in a tree from the perspective of the current node, also called focus . It is represented by a 2-tuple where the first element is the focus and the second element is the metadata/context. The metadata is nil when the focus is the topmost node","ref":"Styler.Zipper.html"},{"type":"function","title":"Styler.Zipper.append_child/2","doc":"Inserts the item as the rightmost child of the node at this zipper, without moving.","ref":"Styler.Zipper.html#append_child/2"},{"type":"function","title":"Styler.Zipper.children/1","doc":"Returns a list of children of the node.","ref":"Styler.Zipper.html#children/1"},{"type":"function","title":"Styler.Zipper.down/1","doc":"Returns the zipper of the leftmost child of the node at this zipper, or nil if no there's no children.","ref":"Styler.Zipper.html#down/1"},{"type":"function","title":"Styler.Zipper.find/3","doc":"Returns a zipper to the node that satisfies the predicate function, or nil if none is found. The optional second parameters specifies the direction , defaults to :next .","ref":"Styler.Zipper.html#find/3"},{"type":"function","title":"Styler.Zipper.insert_child/2","doc":"Inserts the item as the leftmost child of the node at this zipper, without moving.","ref":"Styler.Zipper.html#insert_child/2"},{"type":"function","title":"Styler.Zipper.insert_left/2","doc":"Inserts the item as the left sibling of the node at this zipper, without moving. Raises an ArgumentError when attempting to insert a sibling at the top level.","ref":"Styler.Zipper.html#insert_left/2"},{"type":"function","title":"Styler.Zipper.insert_right/2","doc":"Inserts the item as the right sibling of the node at this zipper, without moving. Raises an ArgumentError when attempting to insert a sibling at the top level.","ref":"Styler.Zipper.html#insert_right/2"},{"type":"function","title":"Styler.Zipper.insert_siblings/2","doc":"Inserts many siblings to the right. Equivalent to Enum . reduce ( siblings , zipper , &amp; Zipper . insert_right ( &amp;2 , &amp;1 ) )","ref":"Styler.Zipper.html#insert_siblings/2"},{"type":"function","title":"Styler.Zipper.left/1","doc":"Returns the zipper of the left sibling of the node at this zipper, or nil.","ref":"Styler.Zipper.html#left/1"},{"type":"function","title":"Styler.Zipper.leftmost/1","doc":"Returns the leftmost sibling of the node at this zipper, or itself.","ref":"Styler.Zipper.html#leftmost/1"},{"type":"function","title":"Styler.Zipper.next/1","doc":"Returns the following zipper in depth-first pre-order.","ref":"Styler.Zipper.html#next/1"},{"type":"function","title":"Styler.Zipper.node/1","doc":"Returns the node at the zipper.","ref":"Styler.Zipper.html#node/1"},{"type":"function","title":"Styler.Zipper.prepend_siblings/2","doc":"Inserts many siblings to the left. Equivalent to Enum . reduce ( siblings , zipper , &amp; Zipper . insert_left ( &amp;2 , &amp;1 ) )","ref":"Styler.Zipper.html#prepend_siblings/2"},{"type":"function","title":"Styler.Zipper.prev/1","doc":"Returns the previous zipper in depth-first pre-order. If it's already at the end, it returns nil.","ref":"Styler.Zipper.html#prev/1"},{"type":"function","title":"Styler.Zipper.remove/1","doc":"Removes the node at the zipper, returning the zipper that would have preceded it in a depth-first walk.","ref":"Styler.Zipper.html#remove/1"},{"type":"function","title":"Styler.Zipper.replace/2","doc":"Replaces the current node in the zipper with a new node.","ref":"Styler.Zipper.html#replace/2"},{"type":"function","title":"Styler.Zipper.replace_children/2","doc":"Returns a new node, given an existing node and new children.","ref":"Styler.Zipper.html#replace_children/2"},{"type":"function","title":"Styler.Zipper.right/1","doc":"Returns the zipper of the right sibling of the node at this zipper, or nil.","ref":"Styler.Zipper.html#right/1"},{"type":"function","title":"Styler.Zipper.rightmost/1","doc":"Returns the rightmost sibling of the node at this zipper, or itself.","ref":"Styler.Zipper.html#rightmost/1"},{"type":"function","title":"Styler.Zipper.root/1","doc":"Walks the zipper all the way up and returns the root node.","ref":"Styler.Zipper.html#root/1"},{"type":"function","title":"Styler.Zipper.skip/2","doc":"Returns the zipper of the right sibling of the node at this zipper, or the next zipper when no right sibling is available. This allows to skip subtrees while traversing the siblings of a node. The optional second parameters specifies the direction , defaults to :next . If no right/left sibling is available, this function returns the same value as next/1 / prev/1 . The function skip/1 behaves like the :skip in traverse_while/2 and traverse_while/3 .","ref":"Styler.Zipper.html#skip/2"},{"type":"function","title":"Styler.Zipper.top/1","doc":"Walks the zipper all the way up and returns the top zipper.","ref":"Styler.Zipper.html#top/1"},{"type":"function","title":"Styler.Zipper.traverse/2","doc":"Traverses the tree in depth-first pre-order calling the given function for each node. If the zipper is not at the top, just the subtree will be traversed. The function must return a zipper.","ref":"Styler.Zipper.html#traverse/2"},{"type":"function","title":"Styler.Zipper.traverse/3","doc":"Traverses the tree in depth-first pre-order calling the given function for each node with an accumulator. If the zipper is not at the top, just the subtree will be traversed.","ref":"Styler.Zipper.html#traverse/3"},{"type":"function","title":"Styler.Zipper.traverse_while/2","doc":"Traverses the tree in depth-first pre-order calling the given function for each node. The traversing will continue if the function returns {:cont, zipper} , skipped for {:skip, zipper} and halted for {:halt, zipper} If the zipper is not at the top, just the subtree will be traversed. The function must return a zipper.","ref":"Styler.Zipper.html#traverse_while/2"},{"type":"function","title":"Styler.Zipper.traverse_while/3","doc":"Traverses the tree in depth-first pre-order calling the given function for each node with an accumulator. The traversing will continue if the function returns {:cont, zipper, acc} , skipped for {:skip, zipper, acc} and halted for {:halt, zipper, acc} If the zipper is not at the top, just the subtree will be traversed.","ref":"Styler.Zipper.html#traverse_while/3"},{"type":"function","title":"Styler.Zipper.up/1","doc":"Returns the zipper of the parent of the node at this zipper, or nil if at the top.","ref":"Styler.Zipper.html#up/1"},{"type":"function","title":"Styler.Zipper.update/2","doc":"Replaces the current node in the zipper with the result of applying fun to the node.","ref":"Styler.Zipper.html#update/2"},{"type":"function","title":"Styler.Zipper.zip/1","doc":"Creates a zipper from a tree node.","ref":"Styler.Zipper.html#zip/1"},{"type":"type","title":"Styler.Zipper.command/0","doc":"","ref":"Styler.Zipper.html#t:command/0"},{"type":"opaque","title":"Styler.Zipper.path/0","doc":"","ref":"Styler.Zipper.html#t:path/0"},{"type":"type","title":"Styler.Zipper.t/0","doc":"","ref":"Styler.Zipper.html#t:t/0"},{"type":"type","title":"Styler.Zipper.tree/0","doc":"","ref":"Styler.Zipper.html#t:tree/0"},{"type":"type","title":"Styler.Zipper.zipper/0","doc":"","ref":"Styler.Zipper.html#t:zipper/0"},{"type":"exception","title":"Styler.StyleError","doc":"Wraps errors raised by Styles during tree traversal.","ref":"Styler.StyleError.html"},{"type":"function","title":"Styler.StyleError.message/1","doc":"Callback implementation for Exception.message/1 .","ref":"Styler.StyleError.html#message/1"},{"type":"extras","title":"Changelog","doc":"","ref":"changelog.html"},{"type":"extras","title":"Changelog - main","doc":"","ref":"changelog.html#main"},{"type":"extras","title":"Changelog - v0.11.3","doc":"Fixes directives: fix infinite loop when encountering @spec import(...) :: ... (Closes #115, h/t @kerryb) with : fix deletion of arrow-less with statements within function invocations","ref":"changelog.html#v0-11-3"},{"type":"extras","title":"Changelog - v0.11.2","doc":"Fixes pipes : fix unpiping do-blocks into variables when the parent expression is a function invocation like a(if x do y end |&gt; z(), b) (Closes #114, h/t @wkirschbaum)","ref":"changelog.html#v0-11-2"},{"type":"extras","title":"Changelog - v0.11.1","doc":"Fixes with : fix with replacement when it's the only child of a do or -&gt; block (Closes #107, h/t @kerryb -- turns out those edge cases did exist in the wild!)","ref":"changelog.html#v0-11-1"},{"type":"extras","title":"Changelog - v0.11.0","doc":"Improvements Comments Styler will no longer make comments jump around in any situation, and will move comments with the appropriate node in all cases but module directive rearrangement (where they'll just be left behind - sorry! we're still working on it). Keep comments in logical places when rewriting if/unless/cond/with (#79, #97, #101, #103) With Statements This release has a slew of improvements for with statements. It's not surprising that there's lots of style rules for with given that just about any case , if , or even cond do could also be expressed as a with . They're very powerful! And with great power... style trivial pattern matches ala lhs &lt;- rhs to lhs = rhs (#86) style _ &lt;- rhs to rhs style keyword , do: to do end rather than wrapping multiple statements in parens style statements all the way to if statements when appropriate (#100) Other Rewrite {Map|Keyword}.merge(single_key: value) to use put/3 instead (#96) Fixes with : various edge cases we can only hope no one's encountered and thus never reported","ref":"changelog.html#v0-11-0"},{"type":"extras","title":"Changelog - v0.10.5","doc":"After being bitten by two of them in a row, Styler's test suite now makes sure that there are no idempotency bugs as part of its tests. In short, we now have assert style(x) == style(style(x)) as part of every test. Sorry for not thinking to include this before :) Fixes alias: fix single-module alias deletion newlines bug comments: ensure all generated nodes always include line meta (#101)","ref":"changelog.html#v0-10-5"},{"type":"extras","title":"Changelog - v0.10.4","doc":"Improvements alias: delete noop single-module aliases ( alias Foo , #87, h/t @mgieger) Fixes pipes: unnest all pipe starts in one pass ( f(g(h(x))) |&gt; j() =&gt; x |&gt; h() |&gt; g() |&gt; f() |&gt; j() , #94, h/t @tomjschuster)","ref":"changelog.html#v0-10-4"},{"type":"extras","title":"Changelog - v0.10.3","doc":"Improvements charlists: leave charlist rewriting to elixir's formatter on elixir &gt;= 1.15 Fixes charlists: rewrite empty charlist to use sigil ( '' =&gt; ~c&quot;&quot; ) pipes: don't blow up extracting fully-qualified macros ( Foo.bar do end |&gt; foo() , #91, h/t @NikitaNaumenko)","ref":"changelog.html#v0-10-3"},{"type":"extras","title":"Changelog - v0.10.2","doc":"Improvements with : remove identity singleton else clause (eg else {:error, e} -&gt; {:error, e} end , else error -&gt; error end )","ref":"changelog.html#v0-10-2"},{"type":"extras","title":"Changelog - v0.10.1","doc":"Fixes Fix function head shrink-failures causing comments to jump into blocks (Closes #67, h/t @APB9785)","ref":"changelog.html#v0-10-1"},{"type":"extras","title":"Changelog - v0.10.0","doc":"Improvements hoist all block-starts to pipes to their own variables (makes styler play better with piped macros) Fixes fix pipes starting with a macro do-block creating invalid ast (#83, h/t @mhanberg)","ref":"changelog.html#v0-10-0"},{"type":"extras","title":"Changelog - v0.9.7","doc":"Fixes rewrite pipes starting with quote blocks like we do with case|if|cond|with blocks (#82, h/t @SteffenDE)","ref":"changelog.html#v0-9-7"},{"type":"extras","title":"Changelog - v0.9.6","doc":"Breaking Change removed mix style task","ref":"changelog.html#v0-9-6"},{"type":"extras","title":"Changelog - v0.9.5","doc":"Fixes fix mistaking Timex.now/1 in a pipe for Timex.now/0 (#66, h/t @sabiwara) Removed style stop rewriting Timex.today/0 given that we allow Timex.today/1 -- too inconsistent.","ref":"changelog.html#v0-9-5"},{"type":"extras","title":"Changelog - v0.9.4","doc":"Improvements if statements: drop else clauses whose body is simply nil","ref":"changelog.html#v0-9-4"},{"type":"extras","title":"Changelog - v0.9.3","doc":"Fixes fix unless a do b else c end rewrites to if not flopping do/else bodies! (#77, h/t @jcowgar) fix pipes styling ranges with steps ( a..b//c ) incorrectly (#76, h/t @cschmatzler)","ref":"changelog.html#v0-9-3"},{"type":"extras","title":"Changelog - v0.9.2","doc":"Fixes fix exception styling module attributes named @def (we confused them with real def s, whoops!) (#75, h/t @randycoulman)","ref":"changelog.html#v0-9-2"},{"type":"extras","title":"Changelog - v0.9.1","doc":"the boolean blocks edition! Improvements auto-fix Credo.Check.Refactor.CondStatements (detects any truthy atom, not just true ) if/unless rewrites: Credo.Check.Refactor.NegatedConditionsWithElse Credo.Check.Refactor.NegatedConditionsInUnless Credo.Check.Refactor.UnlessWithElse","ref":"changelog.html#v0-9-1"},{"type":"extras","title":"Changelog - v0.9.0","doc":"the with statement edition! Improvements Added right-hand-pattern-matching rewrites to for and with left arrow expressions &lt;- (ex: with map = %{} &lt;- foo() =&gt; with %{} = map &lt;- foo ) with statement rewrites, solving the following credo rules Credo.Check.Readability.WithSingleClause Credo.Check.Refactor.RedundantWithClauseResult Credo.Check.Refactor.WithClauses","ref":"changelog.html#v0-9-0"},{"type":"extras","title":"Changelog - v0.8.5","doc":"Fixes Fixed exception when encountering non-arrowed case statements ala case foo, do: unquote(quoted) (#69, h/t @brettinternet, nice)","ref":"changelog.html#v0-8-5"},{"type":"extras","title":"Changelog - v0.8.4","doc":"Fixes Timex related fixes (#66): Rewrite Timex.now/1 to DateTime.now!/1 instead of DateTime.utc_now/1 Only rewrite Timex.today/0 , don't change Timex.today/1","ref":"changelog.html#v0-8-4"},{"type":"extras","title":"Changelog - v0.8.3","doc":"Improvements DateTime rewrites (#62, ht @milmazz) DateTime.compare =&gt; DateTime.{before/after} (elixir &gt;= 1.15) Timex.now =&gt; DateTime.utc_now Timex.today =&gt; Date.utc_today Fixes Pipes: add != , !== , === , and , and or to list of valid infix operators (#64)","ref":"changelog.html#v0-8-3"},{"type":"extras","title":"Changelog - v0.8.2","doc":"Fixes Pipes always de-sugars keyword lists when unpiping them (#60)","ref":"changelog.html#v0-8-2"},{"type":"extras","title":"Changelog - v0.8.1","doc":"Fixes ModuleDirectives doesn't mistake variables for directives (#57, h/t @leandrocp)","ref":"changelog.html#v0-8-1"},{"type":"extras","title":"Changelog - v0.8.0","doc":"Improvements (Bug Fix!?) ModuleDirectives no longer throws comments around a file when hoisting directives up (#53)","ref":"changelog.html#v0-8-0"},{"type":"extras","title":"Changelog - v0.7.14","doc":"Improvements rewrite Logger.warn/1,2 to Logger.warning/1,2 due to Elixir 1.15 deprecation","ref":"changelog.html#v0-7-14"},{"type":"extras","title":"Changelog - v0.7.13","doc":"Fixes don't unpipe single-piped unquote expressions (h/t @elliottneilclark)","ref":"changelog.html#v0-7-13"},{"type":"extras","title":"Changelog - v0.7.12","doc":"Fixes fix 0-arity paren removal on metaprogramming creating uncompilable code (h/t @simonprev)","ref":"changelog.html#v0-7-12"},{"type":"extras","title":"Changelog - v0.7.11","doc":"Fixes fix crash from mix style running plugins as part of formatting (no longer runs formatter plugins) Improvements single-quote charlists are rewritten to use the ~c sigil ( 'foo' -&gt; ~c'foo' ) (h/t @fhunleth) mix style warns the user that Styler is primarily meant to be used as a plugin","ref":"changelog.html#v0-7-11"},{"type":"extras","title":"Changelog - v0.7.10","doc":"Fixes fix crash when encountering single-quote charlists (h/t @fhunleth) Improvements single-quote charlists are rewritten to use the ~c sigil ( 'foo' -&gt; ~c'foo' ) when encountering _ = bar -&gt; , replace it with bar -&gt;","ref":"changelog.html#v0-7-10"},{"type":"extras","title":"Changelog - v0.7.9","doc":"Fixes Fix a toggle state resulting from (ahem, nonsense) code like _ = bar -&gt; encountering ParameterPatternMatching style","ref":"changelog.html#v0-7-9"},{"type":"extras","title":"Changelog - v0.7.8","doc":"Fixes Fix crash trying to remove 0-arity parens from metaprogramming ala def unquote(foo)()","ref":"changelog.html#v0-7-8"},{"type":"extras","title":"Changelog - v0.7.7","doc":"Improvements Rewrite Enum.into/2,3 into Map.new/1,2 when the collectable is %{} or Map.new/0","ref":"changelog.html#v0-7-7"},{"type":"extras","title":"Changelog - v0.7.6","doc":"Fixes Fix crash when single pipe had inner defs (h/t @michallepicki )","ref":"changelog.html#v0-7-6"},{"type":"extras","title":"Changelog - v0.7.5","doc":"Fixes Fix bug from ParameterPatternMatching implementation that re-ordered pattern matching in cond do -&gt; clauses","ref":"changelog.html#v0-7-5"},{"type":"extras","title":"Changelog - v0.7.4","doc":"Features Implement Credo.Check.Readability.PreferImplicitTry Implement Credo.Check.Consistency.ParameterPatternMatching for def|defp|fn|case","ref":"changelog.html#v0-7-4"},{"type":"extras","title":"Changelog - v0.7.3","doc":"Features Remove parens from 0-arity function definitions ( Credo.Check.Readability.ParenthesesOnZeroArityDefs )","ref":"changelog.html#v0-7-3"},{"type":"extras","title":"Changelog - v0.7.2","doc":"Features Rewrite case ... true -&gt; ...; _ -&gt; ... to if statements as well","ref":"changelog.html#v0-7-2"},{"type":"extras","title":"Changelog - v0.7.1","doc":"Features Rewrite case ... true / else -&gt; to be if statements","ref":"changelog.html#v0-7-1"},{"type":"extras","title":"Changelog - v0.7.0","doc":"Features Styler.Style.Simple : Optimize Enum.reverse(foo) ++ bar to Enum.reverse(foo, bar) Styler.Style.Pipes : Rewrite |&gt; (&amp; ...).() to |&gt; then(&amp; ...) ( Credo.Check.Readability.PipeIntoAnonymousFunctions ) Add parens to 1-arity pipe functions ( Credo.Check.Readability.OneArityFunctionInPipe ) Optimize a |&gt; Enum.reverse() |&gt; Enum.concat(enum) to Enum.reverse(a, enum)","ref":"changelog.html#v0-7-0"},{"type":"extras","title":"Changelog - v0.6.1","doc":"Improvements Better error handling: mix format will still format files if a style fails Fixes mix style : only run on .ex and .exs files ModuleDirectives : now expands alias __MODULE__.{A, B} (h/t @adriankumpf )","ref":"changelog.html#v0-6-1"},{"type":"extras","title":"Changelog - v0.6.0","doc":"Features mix style : brought back to life for folks who want to incrementally introduce Styler Fixes Styler.Style.Pipes : include x in y and ^foo (for ecto) as a valid pipe starts work even harder to keep rewrites on one line","ref":"changelog.html#v0-6-0"},{"type":"extras","title":"Changelog - v0.5.2","doc":"Fixes ModuleDirectives : handle dynamic module names Pipes : include Ecto.Query.from and Query.from as valid pipe starts","ref":"changelog.html#v0-5-2"},{"type":"extras","title":"Changelog - v0.5.1","doc":"Improvements Sped up styling just a little bit","ref":"changelog.html#v0-5-1"},{"type":"extras","title":"Changelog - v0.5.0","doc":"Improvements Styler now implements Mix.Task.Format , meaning it is now an Elixir formatter plugin. See the README for new installation &amp; usage instructions Breaking Change! Wooo! the mix style task has been removed","ref":"changelog.html#v0-5-0"},{"type":"extras","title":"Changelog - v0.4.1","doc":"Improvements Pipes rewrites |&gt; Enum.into(%{}[, mapper]) and Enum.into(Map.new()[, mapper]) to Map.new/1,2 calls","ref":"changelog.html#v0-4-1"},{"type":"extras","title":"Changelog - v0.4.0","doc":"Improvements Pipes rewrites some two-step processes into one, fixing these credo issues in pipe chains: Credo.Check.Refactor.FilterCount Credo.Check.Refactor.MapJoin Credo.Check.Refactor.MapInto Fixes ModuleDirectives handles even weirder places to hide your aliases (anonymous functions, in this case) Pipes tries even harder to keep single-pipe rewrites of invocations on one line","ref":"changelog.html#v0-4-0"},{"type":"extras","title":"Changelog - v0.3.1","doc":"Fixes Pipes fixed omission of == as a valid pipe start operator (h/t @peake100 for the issue) fixed rewrite of a |&gt; b , where b was invoked without parenthesis","ref":"changelog.html#v0-3-1"},{"type":"extras","title":"Changelog - v0.3.0","doc":"Improvements Enabled Defs style and overhauled it to properly handles comments Optimized and tweaked ModuleDirectives style Now culls newlines between &quot;groups&quot; of the same directive sorts @behaviour directives orders directives within non defmodule contexts (eg, a def do ) if there's at least one alias|require|use|import Fixes Pipes will try to keep single-pipe rewrites on one line","ref":"changelog.html#v0-3-0"},{"type":"extras","title":"Changelog - v0.2.0","doc":"Improvements Added ModuleDirectives style Note that this is potentially destructive in some rare cases. See moduledoc for more. This supersedes the Aliases style, which has been removed. mix style - reads and writes to stdin/stdout Fixes Pipes style is now aware of unless blocks","ref":"changelog.html#v0-2-0"},{"type":"extras","title":"Changelog - v0.1.1","doc":"Improvements Lots of README tweaking =) Optimized some Zipper operations Added Simple style, replacing the following Credo rule: Credo.Check.Readability.LargeNumbers Fixes Exceptions while parsing code now appropriately render filename rather than nofile:xx Fixed opaque Zipper.path() typespec implementation mismatches (thanks @sega-yarkin) Made ex_doc dev only, removing it as a dependency for users of Styler","ref":"changelog.html#v0-1-1"},{"type":"extras","title":"Changelog - v0.1.0","doc":"Improvements Initial release of Styler Added Aliases style, replacing the following Credo rules: Credo.Check.Readability.AliasOrder Credo.Check.Readability.MultiAlias Credo.Check.Readability.UnnecessaryAliasExpansion Added Pipes style, replacing the following Credo rules: Credo.Check.Readability.BlockPipe Credo.Check.Readability.SinglePipe Credo.Check.Refactor.PipeChainStart Added Defs style (currently disabled by default)","ref":"changelog.html#v0-1-0"},{"type":"extras","title":"Styler","doc":"Styler is an Elixir formatter plugin that's combination of mix format and mix credo , except instead of telling you what's wrong, it just rewrites the code for you to fit its style rules. You can learn more about the history, purpose and implementation of Styler from our talk: Styler: Elixir Style-Guide Enforcer @ GigCity Elixir 2023","ref":"readme.html"},{"type":"extras","title":"Styler - Installation","doc":"Add :styler as a dependency to your project's mix.exs : def deps do [ { :styler , &quot;~&gt; 0.11&quot; , only : [ :dev , :test ] , runtime : false } , ] end Then add Styler as a plugin to your .formatter.exs file [ plugins : [ Styler ] ] And that's it! Now when you run mix format you'll also get the benefits of Styler's definitely-always-right style fixes. Configuration There isn't any! This is intentional. Styler is @adobe's internal Style Guide Enforcer - allowing exceptions to the styles goes against that ethos. Happily, it's open source and thus yours to do with as you will =)","ref":"readme.html#installation"},{"type":"extras","title":"Styler - Features (or as we call them, &quot;Styles&quot;)","doc":"At this point, Styler does a lot. We've catalogued a list of Credo rules that it automatically fixes, but it does some things - like shrinking function heads down to a single line when possible - that Credo doesn't care about. Ultimately, the best way to see what Styler does is to just try it out! What could go wrong? (You're using version control, right?) Credo Rules Styler Replaces If you're using Credo and Styler, we recommend disabling these rules in .credo.exs to save on unnecessary checks in CI. Disabling the rules means updating your .credo.exs depending on your configuration: if you're using checks: %{enabled: [...]} , ensure none of the checks are listed in your enabled checks if you're using checks: %{disabled: [...]} , copy/paste the snippet below into the list if you're using checks: [...] , copy/paste the snippet below into the list and ensure none of the checks appear earlier in the list # Styler Rewrites # # The following rules are automatically rewritten by Styler and so disabled here to save time # Some of the rules have `priority: :high`, meaning Credo runs them unless we explicitly disable them # (removing them from this file wouldn&#39;t be enough, the `false` is required) # # Some rules have a comment before them explaining ways Styler deviates from the Credo rule. # # always expands `A.{B, C}` { Credo.Check.Consistency.MultiAliasImportRequireUse , false } , # including `case`, `fn` and `with` statements { Credo.Check.Consistency.ParameterPatternMatching , false } , { Credo.Check.Readability.AliasOrder , false } , { Credo.Check.Readability.BlockPipe , false } , # goes further than formatter - fixes bad underscores, eg: `100_00` -&gt; `10_000` { Credo.Check.Readability.LargeNumbers , false } , # adds `@moduledoc false` { Credo.Check.Readability.ModuleDoc , false } , { Credo.Check.Readability.MultiAlias , false } , { Credo.Check.Readability.OneArityFunctionInPipe , false } , # removes parens { Credo.Check.Readability.ParenthesesOnZeroArityDefs , false } , { Credo.Check.Readability.PipeIntoAnonymousFunctions , false } , { Credo.Check.Readability.PreferImplicitTry , false } , { Credo.Check.Readability.SinglePipe , false } , # **potentially breaks compilation** - see **Troubleshooting** section below { Credo.Check.Readability.StrictModuleLayout , false } , { Credo.Check.Readability.UnnecessaryAliasExpansion , false } , { Credo.Check.Readability.WithSingleClause , false } , { Credo.Check.Refactor.CaseTrivialMatches , false } , { Credo.Check.Refactor.CondStatements , false } , # in pipes only { Credo.Check.Refactor.FilterCount , false } , # in pipes only { Credo.Check.Refactor.MapInto , false } , # in pipes only { Credo.Check.Refactor.MapJoin , false } , { Credo.Check.Refactor.NegatedConditionsInUnless , false } , { Credo.Check.Refactor.NegatedConditionsWithElse , false } , # allows ecto&#39;s `from { Credo.Check.Refactor.PipeChainStart , false } , { Credo.Check.Refactor.RedundantWithClauseResult , false } , { Credo.Check.Refactor.UnlessWithElse , false } , { Credo.Check.Refactor.WithClauses , false } ,","ref":"readme.html#features-or-as-we-call-them-styles"},{"type":"extras","title":"Styler - Your first Styling","doc":"Speed : Expect the first run to take some time as Styler rewrites violations of styles. Once styled the first time, future styling formats shouldn't take noticeably more time. Troubleshooting: Compilation broke due to Module Directive rearrangement Sometimes naively moving Module Directives around can break compilation. Here's helpers on how to manually fix that and have a happy styling for the rest of your codebase's life. Alias dependency If you have an alias that, for example, a @behaviour relies on, compilation will break after your first run. This requires one-time manual fixing to get your repo in line with Styler's standards. For example, if your code was this: defmodule MyModule do @moduledoc &quot;Implements MyBehaviour!&quot; alias Deeply.Nested.MyBehaviour @behaviour MyBehaviour ... end then Styler will style the file like this, which cannot compile due to MyBehaviour not being defined. defmodule MyModule do @moduledoc &quot;Implements MyBehaviour!&quot; @behaviour MyBehaviour # &lt;------ compilation error, MyBehaviour is not defined! alias Deeply.Nested.MyBehaviour ... end A simple solution is to manually expand the alias with a find-replace-all like: @behaviour MyBehaviour -&gt; @behaviour Deeply.Nested.MyBehaviour . It's important to specify that you only want to find-replace with the @behaviour prefix or you'll unintentially expand MyBehaviour everywhere in the codebase. Module Attribute dependency Another common compilation break on the first run is a @moduledoc that depended on another module attribute which was moved below it. For example, given the following broken code after an initial mix format : defmodule MyGreatLibrary do @moduledoc make_pretty_docs ( @library_options ) @library_options [ ... ] end You can fix the code by moving the static value outside of the module into a naked variable and then reference it in the module. Yes, this is a thing you can do in a .ex file =) library_options = [ ... ] defmodule MyGreatLibrary do @moduledoc make_pretty_docs ( library_options ) @library_options library_options end","ref":"readme.html#your-first-styling"},{"type":"extras","title":"Styler - Thanks &amp; Inspiration","doc":"Sourceror This work was inspired by earlier large-scale rewrites of an internal codebase that used the fantastic tool Sourceror . The initial implementation of Styler used Sourceror, but Sourceror's AST-embedding comment algorithm slows Styler down to the point that it's no longer an appropriate drop-in for mix format . Still, we're grateful for the inspiration Sourceror provided and the changes to the Elixir AST APIs that it drove. The AST-Zipper implementation in this project was forked from Sourceror's implementation. Credo Similarly, this project originated from one-off scripts doing large scale rewrites of an enormous codebase as part of an effort to enable particular Credo rules for that codebase. Credo's tests and implementations were referenced for implementing Styles that took the work the rest of the way. Thanks to Credo &amp; the Elixir community at large for coalescing around many of these Elixir style credos.","ref":"readme.html#thanks-inspiration"}]